using Contracts.DTO;
using Contracts.SearchModels;
using Contracts.Services;
using Contracts.ViewModels;
using Microsoft.AspNetCore.Mvc;
using Services.Support.Exceptions;

namespace Controllers.Controllers;

[Route("api/[controller]")]
[ApiController]
public class ChangeRecordController : ControllerBase
{
    private readonly IChangeRecordService _changeRecordService;

    public ChangeRecordController(IChangeRecordService changeRecordService)
    {
        _changeRecordService = changeRecordService;
    }

    [HttpPost]
    public async Task<ActionResult<ChangeRecordViewModel>> CreateChangeRecord(
        [FromBody] ChangeRecordDto dto)
    {
        try
        {
            var record = await _changeRecordService.Create(dto);
            return CreatedAtAction(nameof(GetChangeRecords), record);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpGet]
    public async Task<ActionResult<IEnumerable<ChangeRecordViewModel>>> GetChangeRecords(
        [FromQuery] ChangeRecordSearch search)
    {
        try
        {
            var records = await _changeRecordService.GetList(search);
            return Ok(records);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpPatch]
    public async Task<ActionResult<ChangeRecordViewModel>> UpdateChangeRecord([FromBody] ChangeRecordDto dto)
    {
        try
        {
            var record = await _changeRecordService.Update(dto);
            return Ok(record);
        }
        catch (EntityNotFoundException ex)
        {
            return NotFound(ex.Message);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpDelete]
    public async Task<ActionResult> DeleteChangeRecord([FromQuery] ChangeRecordSearch search)
    {
        try
        {
            var record = await _changeRecordService.Delete(search);
            return Ok(record);
        }
        catch (EntityNotFoundException ex)
        {
            return NotFound(ex.Message);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }
}