namespace Controllers.Extensions;

using Contracts.DTO;
using Contracts.SearchModels;
using Contracts.Services;
using Contracts.ViewModels;
using Microsoft.AspNetCore.Mvc;
using Services.Support.Exceptions;

[ApiController]
[Route("api/[controller]")]
public class SpendingGroupController : ControllerBase
{
    private readonly ISpendingGroupService _spendingGroupService;

    public SpendingGroupController(ISpendingGroupService spendingGroupService)
    {
        _spendingGroupService = spendingGroupService;
    }
    [HttpGet("{id}")]
    public async Task<ActionResult<SpendingGroupViewModel>> GetSpendingGroup(
        Guid id,
        [FromQuery] SpendingGroupSearch search)
    {
        try
        {
            search ??= new();
            search.Id = id;

            var group = await _spendingGroupService.GetDetails(search);
            return Ok(group);
        }
        catch (EntityNotFoundException ex)
        {
            return NotFound(ex.Message);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpGet]
    public async Task<ActionResult<List<SpendingGroupViewModel>>> GetSpendingGroups()
    {
        try
        {
            var groups = await _spendingGroupService.GetList();
            return Ok(groups);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpGet("filter")]
    public async Task<ActionResult<List<SpendingGroupViewModel>>> GetSpendingGroups(
        [FromQuery] SpendingGroupSearch search)
    {
        try
        {
            var groups = await _spendingGroupService.GetList(search);
            return Ok(groups);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpPost]
    public async Task<ActionResult<SpendingGroupViewModel>> CreateSpendingGroup(
        [FromBody] SpendingGroupDto dto)
    {
        try
        {
            var group = await _spendingGroupService.Create(dto);
            return CreatedAtAction(nameof(GetSpendingGroup), new { id = group.Id }, group);
        }
        catch (EntityNotFoundException ex)
        {
            return NotFound(ex.Message);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpPatch]
    public async Task<ActionResult<SpendingGroupViewModel>> UpdateSpendingGroup([FromBody] SpendingGroupDto dto)
    {
        try
        {
            var group = await _spendingGroupService.Update(dto);
            return Ok(group);
        }
        catch (EntityNotFoundException ex)
        {
            return NotFound(ex.Message);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }

    [HttpDelete]
    public async Task<ActionResult> DeleteSpendingGroup([FromQuery] SpendingGroupSearch search)
    {
        try
        {
            var group = await _spendingGroupService.Delete(search);
            return Ok(group);
        }
        catch (EntityNotFoundException ex)
        {
            return NotFound(ex.Message);
        }
        catch (Exception ex)
        {
            return StatusCode(500, ex.Message);
        }
    }
}